#include <jrnd.h>

#define alleg_mouse_unused
#define alleg_joystick_unused
#define alleg_flic_unused
#define alleg_datafile_unused
#define alleg_math_unused
#define alleg_gui_unused

#include "allegro.h"

// variable defines -- look in stars.cfg for more explanation
int SW = 640;
int SH = 480;
int DEPTH = 8;
int MAX_BACKSTARS = 256;
int MAX_STARS = 512;
int BACK_MAX = 40;
int BACK_MIN = 20;
int vol = 128;
float MAX_SPEED = 0.2;
float MIN_SPEED = 0.02;

float getrndf(int high, int low) {
  return getrnd(high,low);
}

class star {
  private:
    int color;
    long rx, ry;
    float rz;
    float sx, sy;
    float speed;
  public:
    calc() {
      sx = rx / rz + SW / 2;
      sy = ry / rz + SH / 2;
      color = int(rz * 8 - 1);
      rz -= speed;
    }
    star() {
      color = 0;
      rx = 0;
      ry = 0;
      rz = 1.0;
      sx = 0.0;
      sy = 0.0;
      speed = 0.1;
    }
    setnew() { //init a new star
      rx = (getrnd(SW) - SW / 2) * DEPTH / 2;
      ry = (getrnd(SH) - SH / 2) * DEPTH / 2;
      rz = DEPTH;
      speed = getrndf(int(MAX_SPEED * 100), int(MIN_SPEED * 100)) / 100.0;
      sx = rx / rz + SW / 2.0;
      sy = ry / rz + SH / 2.0;
    }
    display() { //Erase old position, calculate, and display new one
      putpixel(screen, int(sx), int(sy), 0);
      calc();
      putpixel(screen, int(sx), int(sy), abs(color - 64));
      if (rz <= 0) {
        putpixel(screen, int(sx), int(sy), 0);
        setnew();
      }
    }
};

void backgroundstars() {
  int d, x, y, color;

  for (d=0; d<=MAX_BACKSTARS; d++) {
    x = getrnd(SW);
    y = getrnd(SH);
    color = getrnd(BACK_MAX,BACK_MIN);
    putpixel(screen, x, y, color);
  }
}

int frame = 0;

void fpscounter(...) {
  frame = 1;
}

END_OF_FUNCTION(fpscounter);

int backredraw = 0;

void backdraw(...) {
  backredraw = 1;
}

END_OF_FUNCTION(backdraw);

void main() {
  DECLARE_COLOR_DEPTH_LIST(
    COLOR_DEPTH_8
  )

  set_config_file("stars.cfg");
  DEPTH = get_config_int("stars", "DEPTH", 8);
  SW = get_config_int("stars", "SW", 640);
  SH = get_config_int("stars", "SH", 480);
  MAX_BACKSTARS = get_config_int("stars", "MAX_BACKSTARS", 256);
  MAX_STARS = get_config_int("stars", "MAX_STARS", 512);
  BACK_MAX = get_config_int("stars", "BACK_MAX", 40);
  BACK_MIN = get_config_int("stars", "BACK_MIN", 20);
  MAX_SPEED = get_config_float("stars", "MAX_SPEED", 0.2);
  MIN_SPEED = get_config_float("stars", "MIN_SPEED", 0.02);
  vol = get_config_int("stars", "volume", 128);

  star *stars; //create the array of stars
  stars = new star[MAX_STARS];

  PALETTE mypal;

  randomize();

  int c = 0;
  for (c=0; c<=MAX_STARS; c++) { //set the array to initial values
    (stars+c)->setnew();
  }

  allegro_init(); //init stuff
  install_keyboard();
  install_timer();
  install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, NULL);

  LOCK_VARIABLE(frame);
  LOCK_FUNCTION(fpscounter);
  LOCK_VARIABLE(backredraw);
  LOCK_FUNCTION(backdraw);

  fade_out(4);
  set_gfx_mode(GFX_AUTODETECT, SW, SH, 0, 0);
  clear(screen);

  for (c=255; c>=0; c--) { //create palette array
    mypal[c].r = c;
    mypal[c].g = c;
    mypal[c].b = c;
  }

  set_palette(mypal);

  int d = 0;

  backgroundstars();
  backgroundstars();
  backgroundstars();
  backgroundstars();

  MIDI* starwars;
  set_volume(-1,vol);
  starwars = load_midi("starwars.mid");
  play_midi(starwars, true);

  install_int(backdraw, 5000);
  install_int(fpscounter, 25);

  do { //update stars
    for (d=0; d<=MAX_STARS; d++)
      (stars+d)->display();
  do {
  } while (!frame);
  frame = 0;
  if (backredraw) {
    backredraw = 0;
    backgroundstars();
  }

  } while (!keypressed());

  fade_out(1);
  for (d=vol;d>=0; d--) {
    set_volume(-1, d);
    rest(20);
  }
  set_volume(-1, 128);
  stop_midi(starwars);
  destroy_midi(starwars);

  delete[] stars;
  allegro_exit(); //shutdown

  exit(0);
}
